#include "message_handler.h"

static struct k_work_delayable attention_blink_work;
static bool Attention = false;

/**
 * Returns true if the specified address is an address of the local element.
 */
static bool address_is_local(struct bt_mesh_model *mod, uint16_t addr)
{
	return bt_mesh_model_elem(mod)->addr == addr;
}

/**
 * Returns true if the provided address is unicast address.
 */
static bool address_is_unicast(uint16_t addr)
{
	return (addr > 0) && (addr <= 0x7FFF);
}

static void handle_message_model_message(struct message_model_cli *cli, struct bt_mesh_msg_ctx *ctx, const uint8_t *msg)
{
	/* Don't print own messages. */
	if (address_is_local(cli->model, ctx->addr)) {
		return;
	}

	printk("<0x%04X>: %s\n", ctx->addr, msg);
}

static void handle_message_model_buffer(struct message_model_cli *cli, struct bt_mesh_msg_ctx *ctx, const uint8_t *buffer, uint16_t len)
{
	if (address_is_local(cli->model, ctx->addr)) {
		return;
	}
	
	//Test print the message
	if(buffer)
		printk("Received buffer with id: %d\n", buffer[0]);

}

/** @brief Works only with the development kit! */
static void attention_blink(struct k_work *work)
{
	static int idx;
	const uint8_t pattern[] = {
		BIT(0) | BIT(1),
		BIT(1) | BIT(2),
		BIT(2) | BIT(3),
		BIT(3) | BIT(0),
	};

	if (Attention) {
		dk_set_leds(pattern[idx++ % ARRAY_SIZE(pattern)]);
		k_work_reschedule(&attention_blink_work, K_MSEC(30));
	} else {
		dk_set_leds(DK_NO_LEDS_MSK);
	}
}

static void attention_on(struct bt_mesh_model *mod)
{
	Attention = true;
	k_work_reschedule(&attention_blink_work, K_NO_WAIT);
}

static void attention_off(struct bt_mesh_model *mod)
{
	/* Will stop rescheduling blink timer */
	Attention = false;
}

static const struct bt_mesh_health_srv_cb health_srv_cb = {
	.attn_on = attention_on,
	.attn_off = attention_off,
};

static struct bt_mesh_health_srv health_srv = {
	.cb = &health_srv_cb,
};

BT_MESH_HEALTH_PUB_DEFINE(health_pub, 0);

static const struct message_model_handlers cli_handlers = {
	.message = handle_message_model_message,
	.buffer = handle_message_model_buffer,
	// .private_message = handle_chat_private_message,
	// .message_reply = handle_chat_message_reply,
};

static struct message_model_cli cli = {
	.handlers = &cli_handlers,
};


static struct bt_mesh_elem elements[] = {
	BT_MESH_ELEM(
		1,
		BT_MESH_MODEL_LIST(
			BT_MESH_MODEL_CFG_SRV,
			BT_MESH_MODEL_HEALTH_SRV(&health_srv, &health_pub)),
        BT_MESH_MODEL_LIST(MESSAGE_MODEL_CLI(&cli))
    )
};

static const struct bt_mesh_comp comp = {
	.cid = COMP_ID,
	.elem = elements,
	.elem_count = ARRAY_SIZE(elements),
};

int set_status(enum message_model_status status)
{
	return message_model_status_set(&cli, status);
}

int send_message(uint8_t* message){
	return message_model_message_send(&cli, message);
}

int send_buffer(uint8_t* buffer, uint16_t len){
	return message_model_buffer_send(&cli, buffer, len);
}


const struct bt_mesh_comp* message_handler_init(){
	k_work_init_delayable(&attention_blink_work, attention_blink);

	return &comp;
}
